/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/* Evolution planner - iCalendar planner backend
 *
 * Copyright (C) Alvaro del Castillo <acs@barrapunto.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Based in part on the http backend.
 */

#include <config.h>
#include <string.h>
#include <unistd.h>
#include <bonobo/bonobo-exception.h>
#include <bonobo/bonobo-moniker-util.h>
#include <libgnome/gnome-i18n.h>
#include <libedataserver/e-xml-hash-utils.h>
#include <libedataserver/e-uid.h>
#include <libecal/e-cal-recur.h>
#include <libecal/e-cal-util.h>
#include <libedata-cal/e-cal-backend-cache.h>
#include <libedata-cal/e-cal-backend-util.h>
#include <libedata-cal/e-cal-backend-sexp.h>
#include "e-cal-backend-planner.h"
#include <libplanner/mrp-application.h>
#include <libplanner/mrp-project.h>
#include <libplanner/mrp-task.h>

/* Private part of the ECalBackendPlanner structure */
struct _ECalBackendPlannerPrivate {
	/* uri to get planner data from */
	gchar      *uri;
	MrpProject *project;

	/* Local/remote mode */
	CalMode mode;

	/* The calendar's default timezone, used for resolving DATE and
	   floating DATE-TIME values. */
	icaltimezone *default_zone;

	/* ECalComponent hash table */
	GHashTable *tasks_comp;
};

#define d(x)

static void e_cal_backend_planner_dispose (GObject *object);
static void e_cal_backend_planner_finalize (GObject *object);

static ECalBackendSyncClass *parent_class;


/* Dispose handler for the file backend */
static void
e_cal_backend_planner_dispose (GObject *object)
{
	ECalBackendPlanner        *cbplanner;
	ECalBackendPlannerPrivate *priv;

	cbplanner = E_CAL_BACKEND_PLANNER (object);
	priv = cbplanner->priv;

	if (G_OBJECT_CLASS (parent_class)->dispose)
		(* G_OBJECT_CLASS (parent_class)->dispose) (object);
}

/* Finalize handler for the file backend */
static void
e_cal_backend_planner_finalize (GObject *object)
{
	ECalBackendPlanner        *cbplanner;
	ECalBackendPlannerPrivate *priv;

	g_return_if_fail (object != NULL);
	g_return_if_fail (E_IS_CAL_BACKEND_PLANNER (object));

	cbplanner = E_CAL_BACKEND_PLANNER (object);
	priv = cbplanner->priv;

	/* Clean up */

	if (priv->uri) {
	        g_free (priv->uri);
		priv->uri = NULL;
	}

	if (priv->tasks_comp) {
		g_hash_table_destroy (priv->tasks_comp);
	}

	g_free (priv);
	cbplanner->priv = NULL;

	if (G_OBJECT_CLASS (parent_class)->finalize)
		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
}

/* Calendar backend methods */

/* Is_read_only handler for the file backend */
/* It will be read and write in the future */
static ECalBackendSyncStatus
e_cal_backend_planner_is_read_only (ECalBackendSync *backend, 
				    EDataCal        *cal, 
				    gboolean        *read_only)
{
	*read_only = TRUE;
	
	return GNOME_Evolution_Calendar_Success;
}

/* FIXME: The only address for the Planner backend is the file 
   for the moment. Later we will have a URI */
static ECalBackendSyncStatus
e_cal_backend_planner_get_cal_address (ECalBackendSync *backend, 
				       EDataCal        *cal, 
				       char           **address)
{
	*address = NULL;

	return GNOME_Evolution_Calendar_Success;
}

static ECalBackendSyncStatus
e_cal_backend_planner_get_ldap_attribute (ECalBackendSync *backend, 
					  EDataCal        *cal, 
					  char           **attribute)
{
	*attribute = NULL;
	
	return GNOME_Evolution_Calendar_Success;
}

static ECalBackendSyncStatus
e_cal_backend_planner_get_alarm_email_address (ECalBackendSync *backend, 
					       EDataCal        *cal, 
					       char           **address)
{
	*address = NULL;
	
	return GNOME_Evolution_Calendar_Success;
}

static ECalBackendSyncStatus
e_cal_backend_planner_get_static_capabilities (ECalBackendSync *backend, 
					       EDataCal        *cal, 
					       char           **capabilities)
{
	*capabilities = g_strdup (CAL_STATIC_CAPABILITY_NO_EMAIL_ALARMS);

	return GNOME_Evolution_Calendar_Success;
}

/* TODO: for the moment we only want to see that it works! */
static ECalComponent *
task_to_cal_component (MrpTask            *task,
		       ECalBackendPlanner *cbgw)
{
	ECalComponent     *comp;
	ECalComponentText  text;
	gchar             *uid, *name;

	comp = e_cal_component_new ();
	e_cal_component_set_new_vtype (comp, E_CAL_COMPONENT_TODO);

	mrp_object_get (task, "eds-uid", &uid, "name", &name, NULL);
	if (!uid) {
		uid = e_uid_new ();
		mrp_object_set (task, "eds-uid", uid, NULL);	
	}
	g_message ("New task found: %s %s", uid, name);
	e_cal_component_set_uid (comp, uid);
	text.value = name;
	text.altrep = NULL;
	e_cal_component_set_summary (comp, &text);

	return comp;	
}


static gboolean
get_planner_tasks_cb (ECalBackendPlanner *cbplanner)
{
	ECalBackendPlannerPrivate *priv;
	icalcomponent_kind         kind;
	GList                     *tasks, *l;

	priv = cbplanner->priv;

	d(g_message ("Getting planner tasks ...\n"));
	g_message ("Getting planner tasks ...");

	tasks = mrp_project_get_all_tasks (priv->project);

	kind = e_cal_backend_get_kind (E_CAL_BACKEND (cbplanner));

	for (l = tasks; l; l = l->next) {
		ECalComponent *comp;
		const gchar   *uid;

		comp = task_to_cal_component (l->data, cbplanner);
		e_cal_component_get_uid (comp, &uid);
		g_hash_table_insert (priv->tasks_comp, g_strdup (uid), comp); 
	}

	d(g_message ("Planner task retrieval done.\n"));

	return FALSE;
}

static void
task_create_uid_property (MrpProject *project)
{	
	MrpProperty *property;

	property = mrp_property_new ("eds-uid", 
				     MRP_PROPERTY_TYPE_STRING,
				     _("Evolution Data Server UID"),
				     _("Identifier used by Evolution Data Server for tasks"),
				     FALSE);
			
	mrp_project_add_property (project, 
				  MRP_TYPE_TASK,
				  property,
				  FALSE);
}

/* Open handler for the file backend */
static ECalBackendSyncStatus
e_cal_backend_planner_open (ECalBackendSync *backend, 
			    EDataCal        *cal, 
			    gboolean         only_if_exists,
			    const gchar     *username, 
			    const gchar     *password)
{
	ECalBackendPlanner        *cbplanner;
	ECalBackendPlannerPrivate *priv;
	GError                    *error = NULL;

	g_message ("Open planner tasks ...");
	
	cbplanner = E_CAL_BACKEND_PLANNER (backend);
	priv = cbplanner->priv;

	priv->uri = "/tmp/test.planner";
	g_message ("Opening fixed planner file: %s", priv->uri);

	priv->mode = CAL_MODE_REMOTE;
	if (mrp_project_load (priv->project, priv->uri, &error)) {
		/* We need a custom property eds-uid in the planner project */
		if (!mrp_project_has_property (priv->project, 
				       MRP_TYPE_RESOURCE, "eds-uid")) {
			task_create_uid_property (priv->project);
		}
		g_idle_add ((GSourceFunc) get_planner_tasks_cb, cbplanner);
		return GNOME_Evolution_Calendar_Success;
	} else {
		g_warning ("Problem opening planner project: %s", error->message);
		g_error_free (error);
		return GNOME_Evolution_Calendar_ObjectNotFound;
	}
}

static ECalBackendSyncStatus
e_cal_backend_planner_remove (ECalBackendSync *backend, 
			      EDataCal        *cal)
{
	ECalBackendPlanner        *cbplanner;
	ECalBackendPlannerPrivate *priv;
	
	cbplanner = E_CAL_BACKEND_PLANNER (backend);
	priv = cbplanner->priv;

	/* e_file_cache_remove (E_FILE_CACHE (priv->cache)); */
	/* FIXME: we need to remove the task from Planner */
	return GNOME_Evolution_Calendar_Success;
}

/* is_loaded handler for the planner backend */
static gboolean
e_cal_backend_planner_is_loaded (ECalBackend *backend)
{
	ECalBackendPlanner        *cbplanner;
	ECalBackendPlannerPrivate *priv;

	cbplanner = E_CAL_BACKEND_PLANNER (backend);
	priv = cbplanner->priv;

	return TRUE;
}

/* is_remote handler for the planner backend */
static CalMode
e_cal_backend_planner_get_mode (ECalBackend *backend)
{
	ECalBackendPlanner        *cbplanner;
	ECalBackendPlannerPrivate *priv;

	cbplanner = E_CAL_BACKEND_PLANNER (backend);
	priv = cbplanner->priv;

	return priv->mode;
}

/* Set_mode handler for the planner backend */
static void
e_cal_backend_planner_set_mode (ECalBackend *backend, 
				CalMode      mode)
{
	ECalBackendPlanner               *cbplanner;
	ECalBackendPlannerPrivate        *priv;
	GNOME_Evolution_Calendar_CalMode  set_mode;

	cbplanner = E_CAL_BACKEND_PLANNER (backend);
	priv = cbplanner->priv;

	switch (mode) {
		case CAL_MODE_LOCAL:
		case CAL_MODE_REMOTE:
			priv->mode = mode;
			set_mode = cal_mode_to_corba (mode);
			break;
		case CAL_MODE_ANY:
			priv->mode = CAL_MODE_REMOTE;
			set_mode = GNOME_Evolution_Calendar_MODE_REMOTE;
			break;
		default:
			set_mode = GNOME_Evolution_Calendar_MODE_ANY;
			break;
	}

	if (set_mode == GNOME_Evolution_Calendar_MODE_ANY)
		e_cal_backend_notify_mode (backend,
					   GNOME_Evolution_Calendar_CalListener_MODE_NOT_SUPPORTED,
					   cal_mode_to_corba (priv->mode));
	else
		e_cal_backend_notify_mode (backend,
					   GNOME_Evolution_Calendar_CalListener_MODE_SET,
					   set_mode);
}

static ECalBackendSyncStatus
e_cal_backend_planner_get_default_object (ECalBackendSync *backend, 
					  EDataCal        *cal, 
					  char           **object)
{
	ECalBackendPlanner        *cbplanner;
	ECalBackendPlannerPrivate *priv;
	icalcomponent             *icalcomp;
	icalcomponent_kind         kind;

	cbplanner = E_CAL_BACKEND_PLANNER (backend);
	priv = cbplanner->priv;

	kind = e_cal_backend_get_kind (E_CAL_BACKEND (backend));
	icalcomp = e_cal_util_new_component (kind);
	*object = g_strdup (icalcomponent_as_ical_string (icalcomp));
	icalcomponent_free (icalcomp);

	return GNOME_Evolution_Calendar_Success;
}

/* Get_object_component handler for the planner backend */
static ECalBackendSyncStatus
e_cal_backend_planner_get_object (ECalBackendSync *backend, 
				  EDataCal        *cal, 
				  const gchar     *uid, 
				  const gchar     *rid, 
				  gchar          **object)
{
	ECalBackendPlanner        *cbplanner;
	ECalBackendPlannerPrivate *priv;
	ECalComponent             *comp = NULL;

	cbplanner = E_CAL_BACKEND_PLANNER (backend);
	priv = cbplanner->priv;

	g_return_val_if_fail (uid != NULL, GNOME_Evolution_Calendar_ObjectNotFound);

	comp = g_hash_table_lookup (priv->tasks_comp, uid); 
	/* comp = e_cal_backend_cache_get_component (priv->cache, uid, rid); */
	/* We take the comp directly from libplanner */
	if (!comp)
		return GNOME_Evolution_Calendar_ObjectNotFound;

	*object = e_cal_component_get_as_string (comp);
	g_free (comp);

	return GNOME_Evolution_Calendar_Success;
}

/* Get_timezone_object handler for the file backend */
static ECalBackendSyncStatus
e_cal_backend_planner_get_timezone (ECalBackendSync *backend, 
				    EDataCal        *cal, 
				    const gchar     *tzid, 
				    char           **object)
{
	ECalBackendPlanner *cbplanner;
	ECalBackendPlannerPrivate *priv;
	icaltimezone *zone;
	icalcomponent *icalcomp;

	cbplanner = E_CAL_BACKEND_PLANNER (backend);
	priv = cbplanner->priv;

	g_return_val_if_fail (tzid != NULL, GNOME_Evolution_Calendar_ObjectNotFound);

	zone = icaltimezone_get_builtin_timezone_from_tzid (tzid);
	if (!zone)
		return GNOME_Evolution_Calendar_ObjectNotFound;

	icalcomp = icaltimezone_get_component (zone);
	if (!icalcomp)
		return GNOME_Evolution_Calendar_InvalidObject;

	*object = g_strdup (icalcomponent_as_ical_string (icalcomp));

	return GNOME_Evolution_Calendar_Success;
}

/* Add_timezone handler for the file backend */
static ECalBackendSyncStatus
e_cal_backend_planner_add_timezone (ECalBackendSync *backend, 
				    EDataCal        *cal, 
				    const gchar     *tzobj)
{
	ECalBackendPlanner *cbplanner;
	ECalBackendPlannerPrivate *priv;

	cbplanner = (ECalBackendPlanner *) backend;

	g_return_val_if_fail (E_IS_CAL_BACKEND_PLANNER (cbplanner), 
			      GNOME_Evolution_Calendar_OtherError);
	g_return_val_if_fail (tzobj != NULL, GNOME_Evolution_Calendar_OtherError);

	priv = cbplanner->priv;

	/* FIXME: add the timezone to the cache */
	return GNOME_Evolution_Calendar_Success;
}

static ECalBackendSyncStatus
e_cal_backend_planner_set_default_timezone (ECalBackendSync *backend, 
					    EDataCal        *cal, 
					    const gchar     *tzid)
{
	ECalBackendPlanner *cbplanner;
	ECalBackendPlannerPrivate *priv;

	cbplanner = E_CAL_BACKEND_PLANNER (backend);
	priv = cbplanner->priv;
	
	/* FIXME */
	return GNOME_Evolution_Calendar_Success;
}

static void
add_comp_to_list (gpointer   key,
		  gpointer   value,
		  GList    **comps)
{
	g_message ("Adding component to list ...");
	*comps = g_list_append (*comps, value);
}



/* Get_objects_in_range handler for the planner backend */
static ECalBackendSyncStatus
e_cal_backend_planner_get_object_list (ECalBackendSync *backend, 
				       EDataCal        *cal, 
				       const gchar     *sexp, 
				       GList          **objects)
{
	ECalBackendPlanner         *cbplanner;
	ECalBackendPlannerPrivate  *priv;
	GList                      *components = NULL, *l = NULL;
	ECalBackendSExp            *cbsexp;

	cbplanner = E_CAL_BACKEND_PLANNER (backend);
	priv = cbplanner->priv;

	g_message ("Getting the object list ...");

	cbsexp = e_cal_backend_sexp_new (sexp);

	*objects = NULL;
	g_hash_table_foreach (priv->tasks_comp, (GHFunc) add_comp_to_list, &components);
	for (l = components; l; l = l->next) {
		if (e_cal_backend_sexp_match_comp (cbsexp, 
						   E_CAL_COMPONENT (l->data), 
						   E_CAL_BACKEND (backend))) {
			*objects = g_list_append (*objects, 
						  e_cal_component_get_as_string (l->data));
		}
	}
	g_list_free (components);
	g_object_unref (cbsexp);

	return GNOME_Evolution_Calendar_Success;
}

/* get_query handler for the planner backend */
static void
e_cal_backend_planner_start_query (ECalBackend  *backend, 
				   EDataCalView *query)
{
	ECalBackendPlanner         *cbplanner;
	ECalBackendPlannerPrivate  *priv;
	GList                      *components = NULL, *l, *objects = NULL;
	ECalBackendSExp            *cbsexp;

	cbplanner = E_CAL_BACKEND_PLANNER (backend);
	priv = cbplanner->priv;

	d(g_message (G_STRLOC ": Starting query (%s)", e_data_cal_view_get_text (query)));

	/* process all components in the cache */
	cbsexp = e_cal_backend_sexp_new (e_data_cal_view_get_text (query));

	objects = NULL;
	/* FIXME: get components from Planner */
	g_hash_table_foreach (priv->tasks_comp, (GHFunc) add_comp_to_list, &components);
	for (l = components; l; l = l->next) {
		if (e_cal_backend_sexp_match_comp (cbsexp, 
						   E_CAL_COMPONENT (l->data), 
						   E_CAL_BACKEND (backend))) {
			objects = g_list_append (objects, 
						 e_cal_component_get_as_string (l->data));
		}
	}

	e_data_cal_view_notify_objects_added (query, (const GList *) objects);

	g_list_free (components);
	g_list_foreach (objects, (GFunc) g_free, NULL);
	g_list_free (objects);
	g_object_unref (cbsexp);

	e_data_cal_view_notify_done (query, GNOME_Evolution_Calendar_Success);
}

/* Get_free_busy handler for the planner backend */
static ECalBackendSyncStatus
e_cal_backend_planner_get_free_busy (ECalBackendSync *backend, 
				     EDataCal        *cal, 
				     GList           *users,
				     time_t           start, 
				     time_t           end, 
				     GList          **freebusy)
{
	ECalBackendPlanner        *cbplanner;
	ECalBackendPlannerPrivate *priv;

	cbplanner = E_CAL_BACKEND_PLANNER (backend);
	priv = cbplanner->priv;

	g_return_val_if_fail (start != -1 && end != -1, 
			      GNOME_Evolution_Calendar_InvalidRange);
	g_return_val_if_fail (start <= end, GNOME_Evolution_Calendar_InvalidRange);

	/* FIXME */
	return GNOME_Evolution_Calendar_Success;
}

/* Get_changes handler for the planner backend */
static ECalBackendSyncStatus
e_cal_backend_planner_get_changes (ECalBackendSync *backend, 
				   EDataCal        *cal, 
				   const char      *change_id,
				   GList          **adds, 
				   GList          **modifies, 
				   GList          **deletes)
{
	ECalBackendPlanner        *cbplanner;
	ECalBackendPlannerPrivate *priv;

	cbplanner = E_CAL_BACKEND_PLANNER (backend);
	priv = cbplanner->priv;

	g_return_val_if_fail (change_id != NULL, GNOME_Evolution_Calendar_ObjectNotFound);

	/* FIXME */
	return GNOME_Evolution_Calendar_Success;
}

/* Discard_alarm handler for the planner backend */
static ECalBackendSyncStatus
e_cal_backend_planner_discard_alarm (ECalBackendSync *backend, 
				     EDataCal        *cal, 
				     const gchar     *uid, 
				     const gchar     *auid)
{
	ECalBackendPlanner        *cbplanner;
	ECalBackendPlannerPrivate *priv;

	cbplanner = E_CAL_BACKEND_PLANNER (backend);
	priv = cbplanner->priv;

	/* FIXME */
	return GNOME_Evolution_Calendar_Success;
}

static ECalBackendSyncStatus
e_cal_backend_planner_create_object (ECalBackendSync *backend, 
				     EDataCal        *cal, 
				     gchar          **calobj, 
				     gchar          **uid)
{
	ECalBackendPlanner        *cbplanner;
	ECalBackendPlannerPrivate *priv;
	
	cbplanner = E_CAL_BACKEND_PLANNER (backend);
	priv = cbplanner->priv;

	return GNOME_Evolution_Calendar_PermissionDenied;
}

static ECalBackendSyncStatus
e_cal_backend_planner_modify_object (ECalBackendSync *backend, 
				     EDataCal        *cal, 
				     const gchar     *calobj, 
				     CalObjModType    mod, 
				     char           **old_object)
{
	ECalBackendPlanner        *cbplanner;
	ECalBackendPlannerPrivate *priv;

	cbplanner = E_CAL_BACKEND_PLANNER (backend);
	priv = cbplanner->priv;

	g_return_val_if_fail (calobj != NULL, GNOME_Evolution_Calendar_ObjectNotFound);

	return GNOME_Evolution_Calendar_PermissionDenied;
}

/* Remove_object handler for the planner backend */
static ECalBackendSyncStatus
e_cal_backend_planner_remove_object (ECalBackendSync *backend, 
				     EDataCal        *cal,
				     const gchar     *uid, 
				     const gchar     *rid,
				     CalObjModType    mod, 
				     gchar          **object)
{
	ECalBackendPlanner        *cbplanner;
	ECalBackendPlannerPrivate *priv;

	cbplanner = E_CAL_BACKEND_PLANNER (backend);
	priv = cbplanner->priv;

	g_return_val_if_fail (uid != NULL, GNOME_Evolution_Calendar_ObjectNotFound);

	return GNOME_Evolution_Calendar_PermissionDenied;
}

/* Update_objects handler for the planner backend. */
static ECalBackendSyncStatus
e_cal_backend_planner_receive_objects (ECalBackendSync *backend, 
				       EDataCal        *cal, 
				       const gchar     *calobj)
{
	ECalBackendPlanner        *cbplanner;
	ECalBackendPlannerPrivate *priv;

	cbplanner = E_CAL_BACKEND_PLANNER (backend);
	priv = cbplanner->priv;

	g_return_val_if_fail (calobj != NULL, GNOME_Evolution_Calendar_InvalidObject);

	return GNOME_Evolution_Calendar_PermissionDenied;
}

static ECalBackendSyncStatus
e_cal_backend_planner_send_objects (ECalBackendSync *backend, 
				    EDataCal        *cal, 
				    const gchar     *calobj, 
				    GList          **users,
				    gchar          **modified_calobj)
{
	ECalBackendPlanner        *cbplanner;
	ECalBackendPlannerPrivate *priv;

	cbplanner = E_CAL_BACKEND_PLANNER (backend);
	priv = cbplanner->priv;

	*users = NULL;
	*modified_calobj = NULL;

	return GNOME_Evolution_Calendar_PermissionDenied;
}

static icaltimezone *
e_cal_backend_planner_internal_get_default_timezone (ECalBackend *backend)
{
	ECalBackendPlanner        *cbplanner;
	ECalBackendPlannerPrivate *priv;

	cbplanner = E_CAL_BACKEND_PLANNER (backend);
	priv = cbplanner->priv;

	return NULL;
}

static icaltimezone *
e_cal_backend_planner_internal_get_timezone (ECalBackend *backend, 
					     const gchar *tzid)
{
	ECalBackendPlanner        *cbplanner;
	ECalBackendPlannerPrivate *priv;
	icaltimezone              *zone;

	cbplanner = E_CAL_BACKEND_PLANNER (backend);
	priv = cbplanner->priv;

	if (!strcmp (tzid, "UTC"))
	        zone = icaltimezone_get_utc_timezone ();
	else {
		zone = icaltimezone_get_builtin_timezone_from_tzid (tzid);
	}

	return zone;
}

/* Object initialization function for the file backend */
static void
e_cal_backend_planner_init (ECalBackendPlanner      *cbplanner, 
			    ECalBackendPlannerClass *class)
{
	ECalBackendPlannerPrivate *priv;

	priv = g_new0 (ECalBackendPlannerPrivate, 1);
	cbplanner->priv = priv;
	priv->tasks_comp = g_hash_table_new_full (NULL, NULL, g_free, NULL);

	priv->project = mrp_project_new (mrp_application_new ());

	priv->uri = NULL;
}

/* Class initialization function for the file backend */
static void
e_cal_backend_planner_class_init (ECalBackendPlannerClass *class)
{
	GObjectClass         *object_class;
	ECalBackendClass     *backend_class;
	ECalBackendSyncClass *sync_class;

	object_class = (GObjectClass *) class;
	backend_class = (ECalBackendClass *) class;
	sync_class = (ECalBackendSyncClass *) class;

	parent_class = (ECalBackendSyncClass *) g_type_class_peek_parent (class);

	object_class->dispose = e_cal_backend_planner_dispose;
	object_class->finalize = e_cal_backend_planner_finalize;

	sync_class->is_read_only_sync = e_cal_backend_planner_is_read_only;
	sync_class->get_cal_address_sync = e_cal_backend_planner_get_cal_address;
 	sync_class->get_alarm_email_address_sync = e_cal_backend_planner_get_alarm_email_address;
 	sync_class->get_ldap_attribute_sync = e_cal_backend_planner_get_ldap_attribute;
 	sync_class->get_static_capabilities_sync = e_cal_backend_planner_get_static_capabilities;
	sync_class->open_sync = e_cal_backend_planner_open;
	sync_class->remove_sync = e_cal_backend_planner_remove;
	sync_class->create_object_sync = e_cal_backend_planner_create_object;
	sync_class->modify_object_sync = e_cal_backend_planner_modify_object;
	sync_class->remove_object_sync = e_cal_backend_planner_remove_object;
	sync_class->discard_alarm_sync = e_cal_backend_planner_discard_alarm;
	sync_class->receive_objects_sync = e_cal_backend_planner_receive_objects;
	sync_class->send_objects_sync = e_cal_backend_planner_send_objects;
 	sync_class->get_default_object_sync = e_cal_backend_planner_get_default_object;
	sync_class->get_object_sync = e_cal_backend_planner_get_object;
	sync_class->get_object_list_sync = e_cal_backend_planner_get_object_list;
	sync_class->get_timezone_sync = e_cal_backend_planner_get_timezone;
	sync_class->add_timezone_sync = e_cal_backend_planner_add_timezone;
	sync_class->set_default_timezone_sync = e_cal_backend_planner_set_default_timezone;
	sync_class->get_freebusy_sync = e_cal_backend_planner_get_free_busy;
	sync_class->get_changes_sync = e_cal_backend_planner_get_changes;

	backend_class->is_loaded = e_cal_backend_planner_is_loaded;
	backend_class->start_query = e_cal_backend_planner_start_query;
	backend_class->get_mode = e_cal_backend_planner_get_mode;
	backend_class->set_mode = e_cal_backend_planner_set_mode;

	backend_class->internal_get_default_timezone = e_cal_backend_planner_internal_get_default_timezone;
	backend_class->internal_get_timezone = e_cal_backend_planner_internal_get_timezone;
}


/**
 * e_cal_backend_planner_get_type:
 * @void: 
 * 
 * Registers the #ECalBackendPlanner class if necessary, and returns the type ID
 * associated to it.
 * 
 * Return value: The type ID of the #ECalBackendPlanner class.
 **/
GType
e_cal_backend_planner_get_type (void)
{
	static GType e_cal_backend_planner_type = 0;

	if (!e_cal_backend_planner_type) {
		static GTypeInfo info = {
                        sizeof (ECalBackendPlannerClass),
                        (GBaseInitFunc) NULL,
                        (GBaseFinalizeFunc) NULL,
                        (GClassInitFunc) e_cal_backend_planner_class_init,
                        NULL, NULL,
                        sizeof (ECalBackendPlanner),
                        0,
                        (GInstanceInitFunc) e_cal_backend_planner_init
                };
		e_cal_backend_planner_type = g_type_register_static (E_TYPE_CAL_BACKEND_SYNC,
								     "ECalBackendPlanner", 
								     &info, 0);
	}

	return e_cal_backend_planner_type;
}
